# Copyright 2001-2021 Arjan Molenaar & Dan Yeaw
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
#
# This license is applicable for all gaphor resources, unless specified
# otherwise.

"""Gaphor is the simple modeling tool written in Python."""

import sys

if sys.platform == "win32":
    from gaphor.windowsshim import gi_init

    gi_init()
