from gaphor.diagram.general import connectors
from gaphor.diagram.general.comment import CommentItem
from gaphor.diagram.general.commentline import CommentLineItem
from gaphor.diagram.general.simpleitem import Box, Ellipse, Line

__all__ = ["CommentItem", "CommentLineItem", "Box", "Ellipse", "Line"]
