# Here, order matters
import gaphor.UML.umloverrides
from gaphor.UML.uml import *
import gaphor.UML.copypaste
import gaphor.UML.deletable
import gaphor.UML.drop
import gaphor.UML.iconname
import gaphor.UML.group
import gaphor.UML.modelinglanguage
from gaphor.UML import recipes
